#--------------------------------Stage Structured Simulation Model  -------------------------------

#  Code for simulating p4t values and logit/inv logit functions needed by runmodelCase
# ------- Simulate p4t values from truncated logistic by transform-uniform method
#-------- T years of n reps for 
p4sim<-function(T,n,pars) {
	x<-c(0:50)/50; 
	m<-pars[1]; s<-pars[2]
	cdf<-(plogis(x,m,s)-plogis(0,m,s))/(plogis(1,m,s)-plogis(0,m,s))
	cdfi<-approxfun(cdf,x); 
	u<-runif(T*n); X<-cdfi(u);
	return( matrix(X,T,n) )
} 

logit<-function(x){log(x/(1-x))}
invlogit<-function(x){exp(x)/(1+exp(x))}

runmodelCase<-function(epars, gregs, gresids, gyreffects, npars, ppars, nyears, nsims2, Release, Harvest, case){
 
#  Model to estimate growth rate using the stage-structured model 

# Inputs----------------------
#  epars = mean and variance for lognormal distribution describing parr-to-smolt survival
#  gregs = regression coefficient for drought year in parr-to-smolt survival regression model
#  gresids = residuals from fit of parr-to-smolt survival regression model
#  gyreffects = estimated year effects from  parr-to-smolt survival regression model
#  npars = smolt-to-adult survival parameters (regression coefficients = columns 1-4, sighat = column 5)
#  ppars = logistic distribution age at spawning parameters 
#  nyears = number of years to simulate
#  nsims2 = number of simulations used to estimate mu
#  Release = hatchery release size in good years (case 2) or in all years (cases 0, 1, 3, 4)
#  Harvest = AEQ harvest rate applied in all years
#  Case = 0  baseline, 1 = increase parr-to-smolt survival, 2 = reduce hatchery release size by 50% in 
#    "bad" survival years, 3=eliminate harvest

#  Runs the model for nyears for nsims2 replicates simultaneously.  
#  Rows index different replicates, columns index years 1:nyears
 
#  Initialize matrices to hold spawner counts and yearly growth rates on log scale
Svect <- matrix(0, nsims2, nyears+5)  #  Spawner counts
lnRts <- matrix(0,nsims2, nyears);  # log running sum of spawner counts
ymu <- matrix(0,nyears,1)  #  mean(lnRts) for each year, used in regression to get mu 

#  Set up vector to hold last 5 years of Ss for each simulation  
Svect[1:nsims2,1:5]<- 10000  # initial number of spawners set to 10000 for all runs (for past 5 years)

#  Generate parr production, parr survival, adult survival for nyears*nsims2

#  Parr Production
etas<-rnorm(nsims2*(nyears+1),epars[1], sqrt(epars[2])) ;  
etas<-matrix(exp(etas),nsims2,nyears+1)

#  parr  to smolt survival rates
pdsi2<-t(surrogate(pdsi, ns=nsims2, FFT=T))
D<-as.matrix((pdsi2<=-3))  # form drought index
D<-D[,1:(nyears+1)] # only need nyears + 1 indices for each simulation	
D<-matrix(as.double(D),nsims2*(nyears+1),1); # stack the indices in one long vector
z <-gregs[1]*D+gyreffects[sample(1:6,nsims2*(nyears+1), replace=T)]+sample(gresids, nsims2*(nyears+1), replace=T); 
gams<-matrix(invlogit(z),nsims2,nyears+1)
gams<-gams + 0.05*gams*(case==1)  # increase parr to smolt surv if case=1

# smolt to adult survival rates
Oci<-as.matrix(t(surrogate(OCI, ns=nsims2, FFT=T)))
Oci<-Oci[,1:(nyears+1)];  #again, only need nyears +1 observations for each replicate
Oci<-matrix(Oci, nsims2*(nyears+1),1)
Good<-as.numeric(Oci >=-1)

#  Unless examining hatchery release strategy, assume constant release size
Release2 <- matrix(Release,nsims2*(nyears+1),1)
if(case==2){Release2[Good==0]<-Release*0.5};  #  Use for hatchery release strategy for case 2
nu<-npars[1]+npars[2]*Release2+npars[3]*Good+npars[4]*Release2*Good+rnorm((nyears+1)*nsims2,0, npars[5]);
nu<-matrix(invlogit(nu),nsims2,nyears+1)

#  Age at spawning parameters
p4<-t(p4sim(nyears+1,nsims2,ppars)); 

#  Now, project the model for nyears.  Project all nsims2 replicates simultaneously
for (i in 1:nyears){
    Svect[,i+5]<- (Svect[,i]* etas[,i] *gams[,i]*nu[,i]*p4[,i]+
    Svect[,i+1]*(1-p4[,i+1])* etas[,i+1] *gams[,i+1]*nu[,i+1])*(1-Harvest)
    Rts<- Svect[,i+1]+Svect[,i+2]+Svect[,i+3]+Svect[,i+4]+ Svect[,i+5]
    lnRts[,i] <- log(Rts)
    ymu[i] <-  mean(lnRts[,i] )     
}

#  Estimate mu 
x<-1:nyears;
mu<-coef(lm(ymu~x))[2]

# Output results
return(mu)
}


